@@ -2,18 +2,21 @@ package Hades::Realm::OO;
 use strict;
 use warnings;
 use base qw/Hades/;
-our $VERSION = 0.03;
+our $VERSION = 0.04;
 
 sub new {
 	my ( $cls, %args ) = ( shift(), scalar @_ == 1 ? %{ $_[0] } : @_ );
 	my $self      = $cls->SUPER::new(%args);
-	my %accessors = ( current_class => {}, is_role => {}, meta => {}, );
+	my %accessors = ( is_role => {}, meta => {}, current_class => {}, );
 	for my $accessor ( keys %accessors ) {
+		my $param
+		    = defined $args{$accessor}
+		    ? $args{$accessor}
+		    : $accessors{$accessor}->{default};
 		my $value
-		    = $self->$accessor(
-			defined $args{$accessor}
-			? $args{$accessor}
-			: $accessors{$accessor}->{default} );
+		    = $self->$accessor( $accessors{$accessor}->{builder}
+			? $accessors{$accessor}->{builder}->( $self, $param )
+			: $param );
 		unless ( !$accessors{$accessor}->{required} || defined $value ) {
 			die "$accessor accessor is required";
 		}
@@ -169,7 +172,7 @@ sub build_class_inheritance {
 		return 'extends' if $1 =~ m/parent|base/;
 		return $params[-1];
 	}
-	elsif ( $params[-2] =~ m/^(with|extends)$/ ) {
+	elsif ( $params[2] && $params[-2] =~ m/^(with|extends)$/ ) {
 		my ( $mg, $last, $ident ) = splice @params, -3;
 		$mg->$last($ident);
 		return $last;
@@ -438,7 +441,8 @@ sub build_has_keywords {
 	    = defined $keywords
 	    ? $keywords
 	    : [
-		qw/is isa required default clearer coerce predicate trigger private/];
+		qw/is isa required default clearer coerce predicate trigger private builder/
+	    ];
 	if ( !defined($keywords) || ( ref($keywords) || "" ) ne "ARRAY" ) {
 		$keywords = defined $keywords ? $keywords : 'undef';
 		die
@@ -617,6 +621,31 @@ sub build_after {
 
 }
 
+sub build_accessor_builder {
+	my ( $self, $name, $content ) = @_;
+	if ( !defined($name) || ref $name ) {
+		$name = defined $name ? $name : 'undef';
+		die
+		    qq{Str: invalid value $name for variable \$name in method build_accessor_builder};
+	}
+	if ( !defined($content) || ref $content ) {
+		$content = defined $content ? $content : 'undef';
+		die
+		    qq{Str: invalid value $content for variable \$content in method build_accessor_builder};
+	}
+
+	return (
+		$content =~ m/^(\w+|1)$/
+		? qq|$content|
+		: qq|sub {
+					my (\$self, \$value) = \@_; 
+					$content
+					return \$value;
+				}|
+	);
+
+}
+
 sub build_accessor_coerce {
 	my ( $self, $name, $content ) = @_;
 	if ( !defined($name) || ref $name ) {
@@ -655,7 +684,7 @@ sub build_accessor_trigger {
 	return q|sub { my ($self, $value) = @_;|
 	    . (
 		$content =~ m/^\w+$/
-		? qq|\$value = __PACKAGE__->$content(\$value);|
+		? qq|\$value = \$self->$content(\$value);|
 		: $content
 	    ) . q|return $value; }|;
 
@@ -864,6 +893,12 @@ call build_after method. Expects param $meta to be a HashRef.
 
 	$obj->build_after($meta)
 
+=head2 build_accessor_builder
+
+call build_accessor_builder method. Expects param $name to be a Str, param $content to be a Str.
+
+	$obj->build_accessor_builder($name, $content)
+
 =head2 build_accessor_coerce
 
 call build_accessor_coerce method. Expects param $name to be a Str, param $content to be a Str.
